//    This file is part of BoobTracker.
//
//    BoobTracker is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    BoobTracker is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with BoobTracker.  If not, see <http://www.gnu.org/licenses/>.
//
//    You may contact the original author at : squalou.jenkins@gmail.com

package net.homelinux.pro.boobTraker;

import android.os.SystemClock;
import android.text.format.Time;
import android.widget.Button;
import android.widget.Chronometer;

public class Boob implements StateTransition {

	private Time lastStartTime = new Time();
	private Time lastEndTime = new Time();
	private String side;
	private Button button;
	private BoobState boobState;
	private String duration; //la durée mise en forme
	private Chronometer chrono;

	public void changeStateTo(BoobState boobState) {
		this.boobState=boobState;
		process();
	}

	public void initStateTo(BoobState boobState) {
		this.boobState=boobState;
		init();
	}

	public void setAvailableState(){
		boobState.available(this);
	}

	public void setProcessingState(){
		boobState.processing(this);
	}
	
	public void setUnavailableState(){
		boobState.unavailable(this);
	}
	
	public int getState(){
		return boobState.getCode();
	}
	
	public Boob(Button button, String side, Chronometer chrono) {
		if (this.lastStartTime==null)this.lastStartTime.setToNow();
		this.button = button;
		this.side=side;
		this.chrono=chrono;
	}

	public void process(){
		boobState.setLastStartTime(this);
		boobState.setLastEndTime(this);
		boobState.setDuration(this);
		boobState.manageChrono(this.chrono);
		displayBoob();
	}

	public void init(){
		
		//reinit the chrono. It normally starts at zero with SystemClock.elapsedRealtime(),
		//we change that to go in the past, thus having a chrono restarting as if never stopped
		Time now=new Time();
		now.setToNow();
		
		boobState.manageChrono(this.chrono, this.lastStartTime.toMillis(false)-now.toMillis(false)+SystemClock.elapsedRealtime());
		displayBoob();
	}


	public void displayBoob() {
		button.setBackgroundResource(boobState.getButtonImage());
	}


	public void setLastStartTime(long lastTimeMillis) {
		this.lastStartTime.set(lastTimeMillis);
	}

	public void setLastEndTime(long lastTimeMillis) {
		this.lastEndTime.set(lastTimeMillis);
	}

	public void setSide(String side) {
		this.side=side;
	}

	public Time getLastStartTime() {
		return this.lastStartTime;
	}

	public Time getLastEndTime() {
		return this.lastEndTime;
	}

	public String getSide() {
		return this.side;
	}

	public String getDuration() {
		return this.duration;
	}

	public void setDuration(String duration) {
		this.duration=duration;
	}	
	
}