//    This file is part of BoobTracker.
//
//    BoobTracker is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    BoobTracker is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with BoobTracker.  If not, see <http://www.gnu.org/licenses/>.
//
//    You may contact the original author at : squalou.jenkins@gmail.com

package net.homelinux.pro.boobTraker;

import java.util.ArrayList;
import java.util.List;

import net.homelinux.pro.boobStates.AvailableBoobState;
import net.homelinux.pro.boobStates.ProcessingBoobState;
import net.homelinux.pro.boobStates.UnavailableBoobState;
import net.homelinux.pro.boobTracker.R;
import android.app.Activity;
import android.content.Intent;
import android.content.SharedPreferences;
import android.graphics.Color;
import android.os.Bundle;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.widget.Button;
import android.widget.Chronometer;
import android.widget.TextView;

public class BoobTracker extends Activity {

	public static final int iLEFT=0;
	public static final int iRIGHT=1;
	//1-iLEFT = iRIGHT et inversement.
	
	public static final BoobState available = new AvailableBoobState();
	public static final BoobState processing = new ProcessingBoobState();
	public static final BoobState unavailable = new UnavailableBoobState();
	
    public static final String PREFS_NAME = "BoobsFile";
    
	private static List<Boob> listBoobs = new ArrayList<Boob>();
    
	private static ListFeeds listFeeds = new ListFeeds(true);

	private static Chronometer chrono;
	
	/** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
    	setContentView(R.layout.main);
        
    }
    
    public void onStart(){
    	super.onStart();
    	listFeeds.loadDataFromFile();
    	updatePreviousFeedsTexts();
    	chrono = (Chronometer) findViewById(R.id.chrono);
    }
    
    
    @Override
    public void onResume(){  
    	super.onResume();
    	
    	//buttons are reinitialized, we must recreate everything
    	
    	final Button leftButton = (Button) findViewById(R.id.leftB);
        final Button rightButton = (Button) findViewById(R.id.rightB);
        
        reload(leftButton,rightButton);
        
      	//API v3 requires Listener declarations, no xml onClick available on buttons.
        //here they are :
  
        leftButton.setOnClickListener(new View.OnClickListener() {
            public void onClick(View v) {
            	buttonClicked(iLEFT);
            }
        });
        
        rightButton.setOnClickListener(new View.OnClickListener() {
            public void onClick(View v) {
            	buttonClicked(iRIGHT);
            }
        });

    }

    
    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        MenuInflater inflater = getMenuInflater();
        inflater.inflate(R.menu.boob_menu, menu);
        return true;
    }

    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
		case R.id.greenM:
			listBoobs.get(iLEFT).setAvailableState();
			listBoobs.get(iRIGHT).setAvailableState();
			break;

		case R.id.viewLog:
			//switch to other activity
            Intent myIntent = new Intent(this.getApplicationContext(), FeedLogs.class);
            startActivityForResult(myIntent, 0);

			break;
		
		default:
			break;
		}
    	
    	return true;
    }
    
    @Override
    protected void onPause(){
       super.onPause();

       // We need an Editor object to make preference changes.
       // All objects are from android.context.Context
       SharedPreferences settings = getSharedPreferences(PREFS_NAME, 0);
       SharedPreferences.Editor editor = settings.edit();
       
       saveBoob(editor, iLEFT);
       saveBoob(editor, iRIGHT);

       // Commit the edits!
       editor.commit();
    }
    

    private void buttonClicked(int boobIndex){

		switch (listBoobs.get(boobIndex).getState()) {
		case BoobConsts.BOOB_AVAILABLE:
			//en cas de besoin : bien placer l'autre en unavailale
			if (listBoobs.get(1-boobIndex).getState() == BoobConsts.BOOB_AVAILABLE) listBoobs.get(1-boobIndex).setUnavailableState();;
			listBoobs.get(boobIndex).setProcessingState();
			break;

		case BoobConsts.BOOB_PROCESSING:
			listBoobs.get(boobIndex).setUnavailableState();
			listBoobs.get(1-boobIndex).setAvailableState();
			listFeeds.addAndStore(
						new Feed(listBoobs.get(boobIndex).getSide(),
								listBoobs.get(boobIndex).getLastStartTime(),
								listBoobs.get(boobIndex).getLastEndTime(),
								listBoobs.get(boobIndex).getDuration())
						);
			updatePreviousFeedsTexts();
			break;
		}
    }
    
    
    private void reload(Button leftButton, Button rightButton){
        listBoobs.clear(); //new button references on resume
        listBoobs.add(new Boob(leftButton, getString(R.string.left), chrono));
        listBoobs.add(new Boob(rightButton, getString(R.string.right), chrono));
        
        // Restore preferences at every resume resume
        SharedPreferences settings = getSharedPreferences(PREFS_NAME, 0);

        loadBoob(settings,iLEFT);
        loadBoob(settings,iRIGHT);
        
    }
    
    
    private void saveBoob(SharedPreferences.Editor editor, Integer boobIndex){
        editor.putLong(boobIndex.toString()+"BoobLastTime", listBoobs.get(boobIndex).getLastStartTime().toMillis(false) );
        editor.putLong(boobIndex.toString()+"BoobEndTime", listBoobs.get(boobIndex).getLastEndTime().toMillis(false));
        editor.putInt(boobIndex.toString()+"BoobState", listBoobs.get(boobIndex).getState());
        editor.putString(boobIndex.toString()+"BoobDuration", listBoobs.get(boobIndex).getDuration() );
        
    }
    
    private void loadBoob(SharedPreferences settings, Integer boobIndex){
        
    	listBoobs.get(boobIndex).setDuration(settings.getString(boobIndex.toString()+"BoobDuration",""));
        listBoobs.get(boobIndex).setLastStartTime(settings.getLong(boobIndex.toString()+"BoobLastTime", 0));
    	listBoobs.get(boobIndex).setLastEndTime(settings.getLong(boobIndex.toString()+"BoobEndTime", 0));
    	
        switch (settings.getInt(boobIndex.toString()+"BoobState",BoobConsts.BOOB_AVAILABLE)) {
		case BoobConsts.BOOB_PROCESSING:
			listBoobs.get(boobIndex).initStateTo(processing);
			break;
		case BoobConsts.BOOB_UNAVAILABLE:
			listBoobs.get(boobIndex).initStateTo(unavailable);			
			break;
		default:
			listBoobs.get(boobIndex).initStateTo(available);
			break;
		}


    }

    /**
     * Blank screen : only used in debug purpose
     */
//    private void clearPreviousFeedsTexts(){
//    	
//    	int idTextView;
//    	
//    	for (int i=1;i<BoobConsts.BOOB_MAX_LISTFEED_SIZE;i++){
//        	
//    		try {
//    			idTextView = R.id.class.getField("previous"+String.valueOf(i)).getInt(0);
//    		} catch (Exception e) {
//    			idTextView=0;
//    			e.printStackTrace();
//    		}
//
//    		TextView feedView = (TextView) findViewById(idTextView);
//    		feedView.setText("");
//    		setTextStyleAsFeed(feedView);
//    		
//    	}
//	
//
//    }
    
    /**
     * Mets a jour les n dernieres infos de l'ecran principal
     */
    private void updatePreviousFeedsTexts(){
    	
    	if (! listFeeds.isEmpty()){
    	
	    	int feedIndex = 1;
	    	boolean dayWritten=false;
	    	int i;

	    	TextView feedView;

	    	for (i=1;i<Math.min(BoobConsts.BOOB_MAX_LISTFEED_SIZE, listFeeds.size()+1);i++){
	        	
	    		feedView = getTextViewPreviousFromIndex(i);
	
	        	if (listFeeds.size()>i-1){
	        		if ( ! dayWritten && (dayChanged(feedIndex) || i==BoobConsts.BOOB_MAX_LISTFEED_SIZE-1 ) ) 
	        			dayWritten=writeDay(feedView,feedIndex-1);
	        		else
	        			dayWritten=updatePrevious(feedView,feedIndex++);
	        	}
	    		
	    	}
	    	
	    	//cas des écrans non remplis : ajouter le premier jour au sommet de la pile
	    	if (i < BoobConsts.BOOB_MAX_LISTFEED_SIZE){
	
	    		feedView = getTextViewPreviousFromIndex(i);
	    		
	    		writeDay(feedView,feedIndex-1);
	    	}
    	}
    }

    private TextView getTextViewPreviousFromIndex(int index){
    	
    	int idTextView;
    	
		try {
			idTextView = R.id.class.getField("previous"+String.valueOf(index)).getInt(0);
		} catch (Exception e) {
			idTextView=0;
			e.printStackTrace();
		}
		
		return (TextView) findViewById(idTextView);
    	
    }
    
    
    private boolean updatePrevious(TextView feedView, int index){
    	
    	feedView.setText(listFeeds.get(listFeeds.size()-index).toStringLog());
    	setTextStyleAsFeed(feedView);
    	return false;
    }

    private boolean writeDay(TextView feedView, int index){
        
    	feedView.setText(listFeeds.get(listFeeds.size()-index).getDateStart().format(BoobConsts.TIMEFORMAT_HEADER));
    	setTextStyleAsHeader(feedView);
    	
    	return true;
    }
    
    private boolean dayChanged(int index){
    	
    	if (index<=1) return false; //security + previous1 should never be a day header 
    	
    	if (listFeeds.get(listFeeds.size()-index).getDateStart().monthDay !=
    		listFeeds.get(listFeeds.size()-index+1).getDateStart().monthDay ) return true;
    	
    	return false;
    }
    
    private void setTextStyleAsFeed(TextView feedView){
    	//restore original color by setting a pure alpha
    	feedView.setBackgroundColor(BoobConsts.BOOB_COLOR_TRANSPARENT);
    	feedView.setTextColor(Color.GRAY);
    }
    
    private void setTextStyleAsHeader(TextView feedView){
    	feedView.setBackgroundColor(BoobConsts.BOOB_COLOR_ORANGE);
    	feedView.setTextColor(Color.BLACK);
    }    
    
}