//    This file is part of BoobTracker.
//
//    BoobTracker is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    BoobTracker is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with BoobTracker.  If not, see <http://www.gnu.org/licenses/>.
//
//    You may contact the original author at : squalou.jenkins@gmail.com

package net.homelinux.pro.boobTraker;

import android.text.format.Time;

public class Feed {

	private Time dateStart=new Time();
	private Time dateEnd=new Time();
	private String duration;
	private String side="";
	private String pause="";

	public Feed(String side, Time dateStart, Time dateEnd, String duration){
		this.side=side;
		this.dateStart=dateStart;
		this.dateEnd=dateEnd;
		this.duration=duration;
		
		this.dateStart.normalize(false);
		this.dateEnd.normalize(false);
	}
	
	public Feed(){
		//empty constructor
	}
	
	public void setSide (String side){
		this.side=side;
	}
	
	public String getSide (){
		return this.side;
	}

	public Time getDateStart() {
		return dateStart;
	}

	public Time getDateEnd() {
		return dateEnd;
	}

	public void setDuration(String duration){
		this.duration=duration;
	}
	public String getDuration() {
		return duration;
	}
	/* 
	 * returns a String representing the Feed, as required by the data file.
	 * 
	 * return "side;dateStartMillis;dateEndMillis;duration;pause;dateStartNice;dateEndNice", datesNice being in "YYYY:MM:DD:HH:MM:SS" format.
	 */
	public String toString(){
		return this.side+";"
			+this.dateStart.format(BoobConsts.TIMEFORMAT_FILE)
			+";"+this.dateEnd.format(BoobConsts.TIMEFORMAT_FILE)
			+";"
			+this.duration+";"
			+this.pause;
	}

	
	public String toStringLog(){
		
		String log = dateStart.format(BoobConsts.TIMEFORMAT_SHORT)
						+"   ("+this.pause+") ";
		
		//pause is en empty string, or 3char, or 6 char. Let's adjust that with tabs
		switch (this.pause.length()) {
		case 0:
			log = log+"\t\t\t\t\t";
			break;
		case 1:
			log = log+"\t\t\t\t";
			break;
		case 2:
			log = log+"\t\t\t\t";
			break;
		case 3:
			log = log+"\t\t\t\t";
			break;
		default:
			log = log+"\t\t";
			break;
		}
		
		log = log +this.duration;

		//duration is usually 3 char, or 6 char. Let's adjust that with tabs
		switch (this.duration.length()) {
		case 0:
			log = log+"\t\t\t\t";
			break;
		case 1:
			log = log+"\t\t\t";
			break;
		case 2:
			log = log+"\t\t\t";
			break;
		case 3:
			log = log+"\t\t\t";
			break;
		default:
			log = log+"\t\t";
			break;
		}

		
		log = log + this.side;
		//usually, duration should be 6 char long too. We enforce that for display reasons. 

		return log;
	}
	
	public String getPause() {
		return pause;
	}

	public void setPause(String pause) {
		this.pause = pause;
	}

}
